Package: detectseparation
Title: Detect and Check for Separation and Infinite Maximum Likelihood
        Estimates
Version: 0.4.0
Authors@R: c(person(given = "Ioannis", family = "Kosmidis", role = c("aut", "cre"), email = "ioannis.kosmidis@warwick.ac.uk", comment = c(ORCID = "0000-0003-1556-0302")),
             person(given = "Dirk", family = "Schumacher", role = "aut", email = "mail@dirk-schumacher.net"),
             person(given = "Florian", family = "Schwendinger", role = "aut", email = "FlorianSchwendinger@gmx.at"),
             person(given = "Kjell", family = "Konis", role = "ctb", email = "kjell.konis@me.com"))
Description: Provides pre-fit and post-fit methods for detecting separation and infinite maximum likelihood estimates in generalized linear models with categorical responses. The pre-fit methods apply on binomial-response generalized liner models such as logit, probit and cloglog regression, and can be directly supplied as fitting methods to the glm() function. They solve the linear programming problems for the detection of separation developed in Konis (2007, <https://ora.ox.ac.uk/objects/uuid:8f9ee0d0-d78e-4101-9ab4-f9cbceed2a2a>) using 'ROI' <https://cran.r-project.org/package=ROI> or 'lpSolveAPI' <https://cran.r-project.org/package=lpSolveAPI>. The post-fit methods apply to models with categorical responses, including binomial-response generalized linear models and multinomial-response models, such as baseline category logits and adjacent category logits models; for example, the models implemented in the 'brglm2' <https://cran.r-project.org/package=brglm2> package. The post-fit methods successively refit the model with increasing number of iteratively reweighted least squares iterations, and monitor the ratio of the estimated standard error for each parameter to what it has been in the first iteration. According to the results in Lesaffre & Albert (1989, <https://www.jstor.org/stable/2345845>), divergence of those ratios indicates data separation.
URL: https://github.com/ikosmidis/detectseparation
BugReports: https://github.com/ikosmidis/detectseparation/issues
Imports: ROI, ROI.plugin.lpsolve, lpSolveAPI, pkgload
Depends: R (>= 3.3.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Suggests: knitr, rmarkdown, covr, brglm2, AER, tinytest,
        ROI.plugin.ecos, ROI.plugin.glpk, ROI.plugin.alabama,
        ROI.plugin.neos
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-04-17 07:32:46 UTC; yiannis
Author: Ioannis Kosmidis [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1556-0302>),
  Dirk Schumacher [aut],
  Florian Schwendinger [aut],
  Kjell Konis [ctb]
Maintainer: Ioannis Kosmidis <ioannis.kosmidis@warwick.ac.uk>
Repository: CRAN
Date/Publication: 2026-04-19 05:10:24 UTC
Built: R 4.5.2; ; 2026-04-19 05:34:24 UTC; unix
