# Generated by roxygen2: do not edit by hand

S3method(check_infinite_estimates,glm)
S3method(plot,inf_check)
S3method(print,detect_infinite_estimates)
S3method(print,detect_separation)
export(checkInfiniteEstimates)
export(check_infinite_estimates)
export(detectInfiniteEstimates)
export(detectSeparation)
export(detectSeparationControl)
export(detect_infinite_estimates)
export(detect_separation)
export(detect_separation_control)
import(ROI.plugin.lpsolve)
importFrom(graphics,matplot)
importFrom(stats,binomial)
importFrom(stats,coef)
importFrom(stats,coefficients)
importFrom(stats,gaussian)
importFrom(stats,nobs)
importFrom(stats,sd)
importFrom(stats,update)
importFrom(stats,vcov)
