# Some convenience names

#' @rdname detect_separation
#' @export
detectSeparation <- detect_separation

#' @rdname detect_separation_control
#' @export
detectSeparationControl <- detect_separation_control

#' @rdname check_infinite_estimates
#' @export
checkInfiniteEstimates <- check_infinite_estimates

#' @rdname detect_infinite_estimates
#' @export
detectInfiniteEstimates <- detect_infinite_estimates
